#ifndef _DAQDATAFORMATEXCEPTION_H
#define _DAQDATAFORMATEXCEPTION_H

#include <execinfo.h>
#include <stdexcept>
#include <string>
#include <sstream>
#include <cstdlib>

#define GENERATE_DATAFORMAT_EXCEPTION(msg) \
  throw daqdataformats::DataFormatException(__FILE__, __LINE__, msg)

namespace daqdataformats {

/**
 * Class to report exceptions in the DAQDataFormats package.
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2012/06/13 17:38:03 $
 */

class DataFormatException : public std::exception {

  public:

    /**
     * Constructs a data format exception instance from the specified
     * file, line number, and message.
     */
    DataFormatException(const std::string &fileName, int line,
                        const std::string &message) {

        // convert the line number to a string
        std::stringstream tmpStream;
        tmpStream << line;
        std::string lineNumber = tmpStream.str();

        // build up the exception text
        _fullText = message + "\n" + "    DataFormatException generated at " +
            fileName + ":" + lineNumber;

        // add some limited stack trace information
        void *array[10];
        size_t size;
        char **strings;
        size_t idx;

        size = backtrace (array, 10);
        strings = backtrace_symbols (array, size);
        tmpStream.seekp(0);  // clear out the stream
        for (idx = 0; idx < size; idx++) {
            tmpStream << "      " << strings[idx] << std::endl;
        }
        free (strings);

        _fullText += "\n    Stack trace:\n" + tmpStream.str();
    }

    /**
     * Destructor.
     */
    virtual ~DataFormatException() throw() { 
        return; 
    }

    /**
     * Returns a description of the exception.
     */
    const char* what() const throw() {
        return _fullText.c_str();
    }

 private:

    /**
     * The full description of the exception.
     */
    std::string _fullText;

};

};  // end of namespace

#endif
