#ifndef RAWCONFIGURATIONTAIL_H
#define RAWCONFIGURATIONTAIL_H

#include "DAQDataFormats/RawDAQData.h"

namespace daqdataformats {

#define RAWCONFIGURATIONTAIL_SIZE 5



    namespace rawconftail {

        // Configuration header delimiter
        static const uint32_t Marker_value[2] = {0x464E4F43, 0x44444E45};

        enum RunHeaderMASKS{
            MASK_Reserved1                  = 0xFFFFFFFF, //! Reserved word 1          (32 bits)
            MASK_Reserved2                  = 0xFFFFFFFF, //! Reserved word 2          (32 bits)
            MASK_MarkerLo1                  = 0xFFFFFFFF, //! Marker lower word 1      (32 bits)
            MASK_MarkerLo2                  = 0xFFFFFFFF, //! Marker lower word 2      (32 bits)
            MASK_CRC                        = 0xFFFFFFFF  //! Check sum for the header (32 bits)

        };

        enum RunHeaderSHIFTS{
            SHIFT_Reserved1                  = 0,  //! Reserved word
            SHIFT_Reserved2                  = 0,  //! Reserved word
            SHIFT_MarkerLo1                  = 0,  //! Marker lower word 1      (32 bits)
            SHIFT_MarkerLo2                  = 0,  //! Marker lower word 2      (32 bits)
            SHIFT_CRC                        = 0   //! CRC                      (32 bits)
        };

        enum RunHeaderWORDS{
            WORD_Reserved1                  = 0, //! Reserved word
            WORD_Reserved2                  = 1, //! Reserved word
            WORD_MarkerLo1                  = 2, //! Marker lower word 1      (32 bits)
            WORD_MarkerLo2                  = 3, //! Marker lower word 2      (32 bits)
            WORD_CRC                        = 4

        };
    }// end of namespace rawconfhdr


  class RawConfigurationTail : public RawDAQData {
    
  public:
    // Constructors
    RawConfigurationTail ();                                    //! The default constructor
    RawConfigurationTail (const RawConfigurationTail& copy_in); //! Deep copy constructor
    RawConfigurationTail (void* configurationheader_buffer);    //! Shallow copy constructor
    /// Destructor
    ~RawConfigurationTail();

    /// Initialization of Data members
    void init();
    
    /// Deep Copy
    void copy(const RawConfigurationTail& copy_in);


    /// Size of MilliSlice _Buffer in words
    uint32_t sizeofdata() const;

    /// Print the MilliSlice structure
    void print() const;

    // Getters
    uint32_t getMarkerLo1() const;
    uint32_t getMarkerLo2() const;
    uint32_t getCRC() const;
    
    // Setters
    bool setMarkerLo1 (uint32_t);
    bool setMarkerLo2 (uint32_t);
    bool setCRC(uint32_t);

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;        //!< Is the buffer internal?

  private:

    /// The actual internal buffer (not a pointer)
    uint32_t  _InternalBuffer[RAWCONFIGURATIONTAIL_SIZE];


  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWCONFIGURATIONTAIL_H */
