#ifndef RAWDATABLOCKHEADER_H
#define RAWDATABLOCKHEADER_H

#include "DAQDataFormats/RawDAQData.h"

#define RAWDATABLOCKHEADER_SIZE 5

namespace daqdataformats {

      namespace datablock {
        
        static const uint32_t DataBlock_Marker = 0xDABC;

        enum DataBlockHeaderMASKS{
            MASK_Marker                   = 0xFFFF0000, //! Marker Mask                        (16 bits)
            MASK_Version                  = 0x00000FFF, //! Version Mask                       (12 bits)
            MASK_DataSize                 = 0xFFFFFFFF, //! Data Size Mask                     (32 bits)
            MASK_NumMicroBlocks           = 0xFFFF0000, //! NBlocks Mask                       (16 bits)
            MASK_BuffId                   = 0x0000FFFF, //! Buffer Node ID Mask                (16 bits)
            MASK_TriggerNumberLo          = 0xFFFFFFFF, //! Trigger Number Mask Low            (32 bits)
            MASK_TriggerNumberHi          = 0x000000FF, //! Trigger Number Mask High           (8 bits)
            MASK_MissingData              = 0x08000000, //! Is DataBlock Missing Data?         (1 bit)
            MASK_MonteCarloFlag           = 0x10000000, //! SIM flag                           (1 bit)
            MASK_CRCCalculationUsed       = 0x20000000, //! Is CRC calculation used?           (1 bit)
            MASK_ConnectionInitialization = 0x40000000, //! Initialization Connection bit Mask (1 bit)
            MASK_CloseConnection          = 0x80000000  //! Close Connection bit               (1 bit)

        };
        enum DataBlockHeaderSHIFTS{
            SHIFT_Marker                   = 16,
            SHIFT_Version                  = 0,
            SHIFT_DataSize                 = 0,
            SHIFT_NumMicroBlocks           = 16,
            SHIFT_BuffId                   = 0,
            SHIFT_TriggerNumberLo          = 0,
            SHIFT_TriggerNumberHi          = 0,
            SHIFT_MissingData               = 27,
            SHIFT_MonteCarloFlag           = 28,
            SHIFT_CRCCalculationUsed       = 29,
            SHIFT_ConnectionInitialization = 30,
            SHIFT_CloseConnection          = 31
        };
        enum DataBlockHeaderWORDS{
            WORD_Version                  = 0,
            WORD_Marker                   = 0,
            WORD_DataSize                 = 1,
            WORD_NumMicroBlocks           = 2,
            WORD_BuffId                   = 2,
            WORD_TriggerNumberLo          = 3,
            WORD_TriggerNumberHi          = 4,
            WORD_MissingData               = 4,
            WORD_MonteCarloFlag           = 4,
            WORD_CRCCalculationUsed       = 4,
            WORD_ConnectionInitialization = 4,
            WORD_CloseConnection          = 4
        };
     }// end of namespace datablock
    class RawDataBlockHeader: public RawDAQData {
      
    public:
      
      /// Constructors
      RawDataBlockHeader ();                             //! The default constructor
      RawDataBlockHeader (const RawDataBlockHeader& copy_in);  //! Deep copy constructor
      RawDataBlockHeader (void* dadablockheader_buffer); //! Shallow copy constructor
      /// Destructor
      ~RawDataBlockHeader();



      /// Initialization of the Data members
      void init();

      /// Deep Copy
      void  copy(const RawDataBlockHeader& copy_in);

      /// Print the DataBlock Header
      void print() const;

      // Getters
      uint32_t getVersion()                 const;
      uint32_t getMarker()                  const;
      uint32_t getTrigNumLo()               const;
      uint32_t getTrigNumHi()               const;
      uint64_t getTrigNum()                 const;
      uint16_t getNumMicroBlocks()          const;
      uint16_t getBuffId()                  const;
      uint32_t getDataSize()                const;
      bool     getMonteCarloFlag()          const;
      bool     isCRCCalculationUsed()       const;
      bool     isConnectionInitialization() const;
      bool     isCloseConnection()          const;
      bool     isMissingData()              const;

      // Setters
      bool setVersion                  (uint32_t);
      bool setMarker                   (uint32_t);
      bool setTrigNumLo                (uint32_t);
      bool setTrigNumHi                (uint32_t);
      bool setTrigNum                  (uint64_t);
      bool setBuffId                   (uint16_t);
      bool setNumMicroBlocks           (uint16_t);
      bool setMonteCarloFlag           (bool);
      bool setCRCCalculationUsed       (bool);
      bool setConnectionInitialization (bool);
      bool setCloseConnection          (bool);
      bool setMissingData              (bool);
      bool setDataSize                 (uint32_t);
      bool addDataSize                 (uint32_t);
      bool advanceNumMicroBlocks();

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;              //!< Is the buffer internal?

      
      /// Size of _Buffer in words
      uint32_t sizeofdata() const;
      
    private:

      /// The actual internal buffer (not a pointer)
      uint32_t  _InternalBuffer[RAWDATABLOCKHEADER_SIZE];
      
    };
  
  
} // END NAMESPACE daqdataformats
#endif /* RAWDATABLOCKHEADER_H */
