#ifndef RAWMICROBLOCKHEADER_H
#define RAWMICROBLOCKHEADER_H

#include "DAQDataFormats/RawDCMChan.h"


#define RAWMICROBLOCKHEADER_SIZE 1

namespace daqdataformats {

    namespace rawmicroblockhdr {

        enum MicroBlockHeaderMASKS{
            MASK_Partition                = 0xFC000000, //! Partition                   ( 6 bits)
            MASK_DestBuffID               = 0x03FF0000, //! Dest Buffer ID Mask         (10 bits)
            MASK_DCMChan                  = 0x0000FFFF,
        };

        enum MicroBlockHeaderSHIFTS{
            SHIFT_Partition                = 26,
            SHIFT_DestBuffID               = 16,
            SHIFT_DCMChan                  = 0,
        };

        enum MicroBlockHeaderWORDS{
            WORD_Partition                = 0,
            WORD_DestBuffID               = 0,
            WORD_DCMChan                  = 0,
        };
    }// end of namespace rawmicroblockhdr


  class RawMicroBlockHeader: public RawDAQData {

  public:

    // Constructors
    RawMicroBlockHeader();                                   //! The default constructor
    RawMicroBlockHeader(const RawMicroBlockHeader& copy_in); //! Deep copy constructor
    RawMicroBlockHeader(void* millisliceheader_buffer);      //! Shallow copy constructor
    /// Destructor
    ~RawMicroBlockHeader();

    /// Initialization of the data members
    void init();

    /// Deep Copy
    void  copy(const RawMicroBlockHeader& copy_in);

    /// Print the MilliSlice Header
    void print() const;

    /// Getters
    RawDCMChan*           getDCMChan();
    uint32_t              getDestBuffID()  const;
    uint32_t              getPartition()  const;
    uint32_t              getDiblockId() const;
    uint32_t              getDCMId() const;
    uint32_t              getDetId() const;
    uint32_t              getGlobalDCMID() const;

    // Setters
    bool setDiblockId                (uint32_t);
    bool setDCMId                    (uint32_t);
    bool setDetId                    (uint32_t);
    bool setGlobalDCMID              (uint32_t);
    bool setDestBuffID               (uint32_t);
    bool setPartition                (uint32_t);

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;              //!< Is the buffer internal?


    /// MilliSlice Header size in words
    uint32_t sizeofdata() const;

  private:

    RawDCMChan   _DCMChan;      //! DCM identifier

    /// The actual internal buffer (not a pointer)
    uint32_t  _InternalBuffer[RAWMICROBLOCKHEADER_SIZE];
  };


} // END NAMESPCE daqdataformats
#endif /* RAWMICROBLOCKHEADER_H */
