#ifndef RAWNANOSLICEHEADER_H
#define RAWNANOSLICEHEADER_H

#include "DAQDataFormats/RawDAQData.h"

#define RAWNANOSLICEHEADER_SIZE 1

namespace daqdataformats {

    //! Data structure for holding the unpacked FEB nanoslice header information
  //! Data structure for holding all the nanoslice data of interest
    typedef struct {
       //uint32_t PAD3;
       uint32_t FrameSequenceNumber;
       //uint32_t PAD5;
       uint32_t PixelAddress;
       uint32_t FEBStatus;
       uint32_t FEBLinkID;
       bool DebugMode;
       bool DataPresent;
       bool LinkStatus;
     } NanoSliceHeader;


    namespace rawnanohdr {

        enum NanoSliceHeaderMASKS{
            MASK_LS           = 0x00000001, //! Link Status                         (1 bit)
            MASK_DP           = 0x00000002, //! Denis Perevalov Present/Absent flag (1 bit)
            MASK_DM           = 0x00000004, //! Debug Mode                          (1 bit)
            MASK_FEBId        = 0x000001F8, //! FEB ID                              (6 bit)
            MASK_FEBStatus    = 0x0000FE00, //! FEB Status                          (7 bit)
            MASK_PixelAddress = 0x001F0000, //! Pixel Address                       (5 bit)
            MASK_SEQ          = 0x1C000000, //! Sequence number                     (3 bit)
            MASK_Pad3         = 0xE0000000, //! Pad3                                (3 bit)
	    MASK_DAQ_ENABLED  = 0x00000200, //! DAQ Enabled Status Bit              (1 bit)
	    MASK_BUFF_EMPTY   = 0x00000400, //! APD Data Buffer Empty               (1 bit)
	    MASK_BUFF_FULL    = 0x00000800, //! APD Data Buffer Full                (1 bit)
	    MASK_COMM_ERR     = 0x00001000, //! FEB Communications Error            (1 bit)
	    MASK_PAKT_ERR     = 0x00002000, //! FEB Packet Error                    (1 bit)
	    MASK_OVERFLOW_ERR = 0x00004000, //! FEB Data Buff Overflow Error        (1 bit)
	    MASK_ADC_ERR      = 0x00008000, //! FEB ADC Error                       (1 bit)
	};

          enum NanoSliceHeaderSHIFTS{
            SHIFT_LS           = 0,
            SHIFT_DP           = 1,
            SHIFT_DM           = 2,
            SHIFT_FEBId        = 3,
            SHIFT_FEBStatus    = 9,
            SHIFT_PixelAddress = 16,
            SHIFT_SEQ          = 26,
            SHIFT_Pad3         = 29,
	    SHIFT_DAQ_ENABLED  = 9,
	    SHIFT_BUFF_EMPTY   = 10,
	    SHIFT_BUFF_FULL    = 11,
	    SHIFT_COMM_ERR     = 12,
	    SHIFT_PAKT_ERR     = 13,
	    SHIFT_OVERFLOW_ERR = 14,
	    SHIFT_ADC_ERR      = 15,
	  };
        }
  
  class RawNanoSliceHeader: public RawDAQData {
    
  public:

    // Constructors
    RawNanoSliceHeader();                             //! The default constructor
    RawNanoSliceHeader (NanoSliceHeader Header);      //! Constructor with Data initialization
    RawNanoSliceHeader(const RawNanoSliceHeader& copy_in);  //! Deep copy constructor
    RawNanoSliceHeader(void* nanosliceheader_buffer); //! Shallow copy constructor

    /// Destructor
    ~RawNanoSliceHeader();

    /// Initialization of Data members
    void init();
    void initWithData(NanoSliceHeader Parameters);

    /// Deep Copy
    void  copy(const RawNanoSliceHeader& copy_in);
    void  copy(void* copy_in);

    /// Print the NanoSlice Header
    void print() const;

    //Getters
    bool         getDataPresent()  const;
    bool         getLinkStatus()   const;
    bool         getDebugMode()    const;
    uint32_t     getFEBId()        const;
    uint32_t     getFEBStatus()    const;
    uint32_t     getPixelAddr()    const;
    uint32_t     getSequence()     const;

    // Status Bit get methods
    bool         getDAQEnabled()     const;
    bool         getBufferEmpty()    const;
    bool         getBufferFull()     const;
    bool         getCommError()      const;
    bool         getPacketError()    const;
    bool         getOverflowError()  const;
    bool         getADCError()       const;

    //Setters
    bool setDataPresent  (bool);
    bool setLinkStatus   (bool);
    bool setDebugMode    (bool);
    bool setFEBId        (uint32_t);
    bool setFEBStatus    (uint32_t);
    bool setPixelAddr    (uint32_t);
    bool setSequence     (uint32_t);
    bool setPad3         (uint32_t);
    bool setParameters   (NanoSliceHeader); //All Parameters
    bool advanceSequence ();

    bool setBufferSource (void*);    //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();        //!< Reset the class to use its internal buffer
    bool clear();                    //!< Clear the buffer
    bool reset();                    //!< Reset the class data
    bool isInternalBuffer() const;   //!< Is the buffer internal?

    uint32_t sizeofdata() const;    //! Size of Data

  private:
      /// The actual internal buffer (not a pointer). Used only in the constructor.
      uint32_t  _InternalBuffer[RAWNANOSLICEHEADER_SIZE];
  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWNANOSLICEHEADER_H */
