#include "DAQDataFormats/RawConfigurationHeader.h"
#include "DAQDataFormats/DataFormatException.h"
#include <boost/lexical_cast.hpp>
#include <stdio.h>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif

using namespace daqdataformats;
using namespace rawconfhdr;
//------------------------------------------------------------------------------
// Default constructor
RawConfigurationHeader::RawConfigurationHeader(){

    // Set pointers to point to the internal buffer
    this->resetBufferSource();

    this->init();
}
//------------------------------------------------------------------------------
RawConfigurationHeader::RawConfigurationHeader(const RawConfigurationHeader& copy_in){
   this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawConfigurationHeader::RawConfigurationHeader(void* rawconfigurationheader_buffer){

    // First, initialize the Internal Buffer
            // Set pointers to point to the internal buffer
            this->resetBufferSource();

            // Initialize it
            this->init();

    // Now set the pointer to the external buffer
    this->setBufferSource(rawconfigurationheader_buffer);

}
//------------------------------------------------------------------------------
// Destructor
RawConfigurationHeader::~RawConfigurationHeader() { }

//------------------------------------------------------------------------------
// Initialization of Data members
void RawConfigurationHeader::init(){

    /// Zero out
    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        _InternalBuffer[i] = 0;

    this->setMarkerHi1(Marker_value[0]);
    this->setMarkerHi2(Marker_value[1]);
}

//------------------------------------------------------------------------------
void RawConfigurationHeader::copy(const RawConfigurationHeader& copy_in){

      // Copy the data from one buffer to the other
    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // Setup the main buffer pointer
    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in._Buffer);
}

//------------------------------------------------------------------------------
//Getters
uint32_t RawConfigurationHeader::getMarkerHi1()                  const {return BITFIELDGET(RAWBUFF32[WORD_MarkerHi1]                 , MASK_MarkerHi1                 , SHIFT_MarkerHi1);}
uint32_t RawConfigurationHeader::getMarkerHi2()                  const {return BITFIELDGET(RAWBUFF32[WORD_MarkerHi2]                 , MASK_MarkerHi2                 , SHIFT_MarkerHi2);}
uint32_t RawConfigurationHeader::getConfigurationHeaderVersion() const {return BITFIELDGET(RAWBUFF32[WORD_ConfigurationHeaderVersion], MASK_ConfigurationHeaderVersion, SHIFT_ConfigurationHeaderVersion);}
uint32_t RawConfigurationHeader::getDetectorID()                 const {return BITFIELDGET(RAWBUFF32[WORD_DetectorID]                , MASK_DetectorID                , SHIFT_DetectorID);}
uint32_t RawConfigurationHeader::getConfigurationBlockSize()     const {return BITFIELDGET(RAWBUFF32[WORD_ConfigurationBlockSize]    , MASK_ConfigurationBlockSize    , SHIFT_ConfigurationBlockSize);}
uint32_t RawConfigurationHeader::getResourceManagerID()          const {return BITFIELDGET(RAWBUFF32[WORD_ResourceManagerID]         , MASK_ResourceManagerID         , SHIFT_ResourceManagerID);}
uint32_t RawConfigurationHeader::getRunNumber()                  const {return BITFIELDGET(RAWBUFF32[WORD_RunNumber]                 , MASK_RunNumber                 , SHIFT_RunNumber);}
uint32_t RawConfigurationHeader::getRunType()                    const {return BITFIELDGET(RAWBUFF32[WORD_RunType]                   , MASK_RunType                   , SHIFT_RunType);}
uint32_t RawConfigurationHeader::getNameSize()                   const {return BITFIELDGET(RAWBUFF32[WORD_NameSize]                  , MASK_NameSize                  , SHIFT_NameSize);}
uint32_t RawConfigurationHeader::getPartition()                  const {return BITFIELDGET(RAWBUFF32[WORD_Partition]                 , MASK_Partition                 , SHIFT_Partition);}
uint32_t RawConfigurationHeader::getSubrun()                     const {return BITFIELDGET(RAWBUFF32[WORD_Subrun]                    , MASK_Subrun                    , SHIFT_Subrun);}
uint32_t RawConfigurationHeader::getSystemNumber()               const {return BITFIELDGET(RAWBUFF32[WORD_SystemNumber]              , MASK_SystemNumber              , SHIFT_SystemNumber);}
uint32_t RawConfigurationHeader::getSystemVersionIDX()           const {return BITFIELDGET(RAWBUFF32[WORD_SystemVersionIDX]          , MASK_SystemVersionIDX          , SHIFT_SystemVersionIDX);}

//------------------------------------------------------------------------------
//Setters
bool RawConfigurationHeader::setMarkerHi1(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_MarkerHi1], w, MASK_MarkerHi1, SHIFT_MarkerHi1);
    return true;
}

bool RawConfigurationHeader::setMarkerHi2(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_MarkerHi2], w, MASK_MarkerHi2, SHIFT_MarkerHi2);
    return true;
}
bool RawConfigurationHeader::setConfigurationHeaderVersion(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_ConfigurationHeaderVersion], w, MASK_ConfigurationHeaderVersion, SHIFT_ConfigurationHeaderVersion);
    return true;
}
bool RawConfigurationHeader::setDetectorID(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_DetectorID], w, MASK_DetectorID, SHIFT_DetectorID);
    return true;
}
bool RawConfigurationHeader::setConfigurationBlockSize(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_ConfigurationBlockSize], w, MASK_ConfigurationBlockSize, SHIFT_ConfigurationBlockSize);
    return true;
}

bool RawConfigurationHeader::setResourceManagerID(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_ResourceManagerID], w, MASK_ResourceManagerID, SHIFT_ResourceManagerID);
    return true;
}
bool RawConfigurationHeader::setRunNumber(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_RunNumber], w, MASK_RunNumber, SHIFT_RunNumber);
    return true;
}
bool RawConfigurationHeader::setRunType(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_RunType], w, MASK_RunType, SHIFT_RunType);
    return true;
}
bool RawConfigurationHeader::setNameSize(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_NameSize], w, MASK_NameSize, SHIFT_NameSize);
    return true;
}
bool RawConfigurationHeader::setPartition(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_Partition], w, MASK_Partition, SHIFT_Partition);
    return true;
}
bool RawConfigurationHeader::setSubrun(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_Subrun], w, MASK_Subrun, SHIFT_Subrun);
    return true;
}
bool RawConfigurationHeader::setSystemNumber(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_SystemNumber], w, MASK_SystemNumber, SHIFT_SystemNumber);
    return true;
}
bool RawConfigurationHeader::advanceSystemNumber(){ return this->setSystemNumber(this->getSystemNumber() + 1);}
bool RawConfigurationHeader::setSystemVersionIDX(uint32_t w) {
    BITFIELDSET(RAWBUFF32[WORD_SystemVersionIDX], w, MASK_SystemVersionIDX, SHIFT_SystemVersionIDX);
    return true;
}



bool RawConfigurationHeader::setBufferSource(void* pointer){
        _Buffer = pointer;
        return true;
}
bool RawConfigurationHeader::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
}

// clears all data in the buffer
bool RawConfigurationHeader::clear(){


        /// Loop over the buffer and zero it
        for(uint32_t i = 0; i < this->sizeofdata(); ++i)
            RAWBUFF32[i] = 0;

        return true;
}
bool RawConfigurationHeader::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
}
bool RawConfigurationHeader::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
}



//------------------------------------------------------------------------------
// Printing the rawconfigurationheader structure
void RawConfigurationHeader::print() const{
    printf("\n---RawConfigurationHeader---------------------------------------------size = %u words\n",this->sizeofdata());

    printf(" MarkerHi1=%u"                 , this->getMarkerHi1());
    printf(" MarkerHi2=%u"                 , this->getMarkerHi2());
    printf(" ConfigurationHeaderVersion=%u", this->getConfigurationHeaderVersion());
    printf(" DetectorID=%u"                , this->getDetectorID());
    printf(" ConfigurationBlockSize=%u"    , this->getConfigurationBlockSize());
    printf(" ResourceManagerID=%u"         , this->getResourceManagerID());
    printf(" RunNumber=%u"                 , this->getRunNumber());
    printf(" RunType=%u"                   , this->getRunType());
    printf(" NameSize=%u"                  , this->getNameSize());
    printf(" Partition=%u"                 , this->getPartition());
    printf(" Subrun=%u"                    , this->getSubrun());
    printf(" SystemNumber=%u"              , this->getSystemNumber());
    printf(" SystemVersionIDX=%u"          , this->getSystemVersionIDX());
   
/*
    std::cout<<"\nRawConfigurationHeader Data RAW 32bit WORDs (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        this->printWord(i,false);
        printf(" ");
    }

    std::cout<<"\nrawconfigurationheader Data RAW 32bit Bits (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
            std::cout<<"\n    ";
            this->printWord(i,true);
    }
*/
}

//------------------------------------------------------------------------------
// Size of the rawconfigurationheader _Buffer in words
uint32_t RawConfigurationHeader::sizeofdata() const {

    return RAWCONFIGURATIONHEADER_SIZE;
}
