#include "DAQDataFormats/RawConfigurationSystemID.h"
#include "DAQDataFormats/DataFormatException.h"
#include <boost/lexical_cast.hpp>
#include <stdio.h>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif

using namespace daqdataformats;
using namespace rawconfsysid;
//------------------------------------------------------------------------------
// Default constructor
RawConfigurationSystemID::RawConfigurationSystemID(uint32_t size){

    // Allocate the internal Buffer
    _InternalBuffer.assign(size, 0);

    this->init();
}
//------------------------------------------------------------------------------
RawConfigurationSystemID::RawConfigurationSystemID(const RawConfigurationSystemID& copy_in){
   this->copy(copy_in);
}

//------------------------------------------------------------------------------
// Destructor
RawConfigurationSystemID::~RawConfigurationSystemID() { }

//------------------------------------------------------------------------------
// Initialization of Data members
void RawConfigurationSystemID::init(){


    // Zero out first words
    for(uint32_t i = 0; i < (uint32_t)_InternalBuffer.size(); ++i)
        _InternalBuffer[i] = 0;

    _Buffer = (void*)(&_InternalBuffer[0]);
    
    _SystemNumber = 0;
}

//------------------------------------------------------------------------------
void RawConfigurationSystemID::copy(const RawConfigurationSystemID& copy_in){

        // Set the maximum size
    _SystemNumber = copy_in._SystemNumber;

    // Setup the internal buffer
    _InternalBuffer.reserve(copy_in._InternalBuffer.size());

      // Copy the data from one buffer to the other
    for(uint32_t i = 0; i < (uint32_t)_InternalBuffer.size(); ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    _Buffer = (void*)(&_InternalBuffer[0]);

}


//------------------------------------------------------------------------------
// Add a MicroSlice into the rawconfigurationheader changing the Header appropriately
bool RawConfigurationSystemID::addSystem(uint32_t sys_id, uint32_t sys_version){


   ++_SystemNumber;

   // Set the system fields
   this->setDAQSystemID(this->getSystemNumber(), sys_id);
   this->setDAQSystemVersion(this->getSystemNumber(), sys_version);

   return true;
}

//------------------------------------------------------------------------------
// This is a general method to read the microslice from rawconfigurationheader_buffer to _Buffer
void* RawConfigurationSystemID::readData(void*){

    std::cerr<<"\nCannot read the RawConfigurationNamespace without the given size";


    //Now, that we know the size of the Configuration Header, read the rest
    return 0;
}

//------------------------------------------------------------------------------
void* RawConfigurationSystemID::readData(void* void_buffer, uint32_t size){

  uint32_t* buffer = (uint32_t*)void_buffer;

  if(buffer == 0){return 0;}

  for(uint32_t i = 0; i < size; ++i )
    RAWBUFF32[i] = buffer[i];

  _SystemNumber = size;


  // Return the address of the buffer advanced by 1 word;
  return (void*)(buffer + size);
}


//------------------------------------------------------------------------------
void* RawConfigurationSystemID::readData(void* buffer_void, uint32_t size, uint32_t _Buffer_start_position){

  uint32_t* buffer = (uint32_t*)buffer_void;

  if(buffer == 0){return 0;}

  for(uint32_t i = 0; i < size; ++i )
    RAWBUFF32[i+_Buffer_start_position] = buffer[i];

  // Return the address of the buffer advanced by 1 word;
  return (void*)(buffer+size);
}
//------------------------------------------------------------------------------
//Getters
uint32_t RawConfigurationSystemID::getSystemNumber()  const {return _SystemNumber;}
uint32_t RawConfigurationSystemID::getDAQSystemVersion(uint32_t isystem) const{

    // Check if the requested system number is too low
    if(isystem == 0){
        std::cout<<"\nWARNING from RawConfigurationSystemID::getDAQSystemVersion! "
                 <<"That the hell? Why are you requesting the 0-th system?\n"
                 <<"Oh I see. You forgot that we count everything from 1... Check your code.\n";
        return 0;
    }


    // Check if the requested system number is too high
    if(isystem > this->getSystemNumber()){
        std::cout<<"\nWARNING from RawConfigurationSystemID::getDAQSystemVersion! "
                 <<"The system number you are asking is too high. "
                 <<"The requested system number  = "<<isystem
                 <<", whereas only "<< this->getSystemNumber()
                 <<" are available in this ConfigurationHeader.\n";
        return 0;
    }

    uint32_t word = (isystem - 1) * SYSTEM_SIZE;

    return BITFIELDGET(RAWBUFF32[word], MASK_DAQSystemVersion, SHIFT_DAQSystemVersion);
}
uint32_t RawConfigurationSystemID::getDAQSystemID(uint32_t isystem) const {

    // Check if the requested system number is too low
    if(isystem == 0){
        std::cout<<"\nWARNING from RawConfigurationSystemID::getDAQSystemID! "
                 <<"That the hell? Why are you requesting the 0-th system?\n"
                 <<"Oh I see. You forgot that we count everything from 1... Check your code.\n";
        return 0;
    }

    // Check if the requested system number is too high
    if(isystem > this->getSystemNumber()){
        std::cout<<"\nWARNING from RawConfigurationSystemID::getDAQSystemID! "
                 <<"The system number you are asking is too high. "
                 <<"The requested system number  = "<<isystem
                 <<", whereas only "<< this->getSystemNumber()
                 <<" are available in this ConfigurationHeader.\n";
        return 0;
    }

    uint32_t word = (isystem - 1) * SYSTEM_SIZE;

    return BITFIELDGET(RAWBUFF32[word], MASK_DAQSystemID, SHIFT_DAQSystemID);
}

bool RawConfigurationSystemID::setSystemNumber(uint32_t w) {
    _SystemNumber = w;
    return true;
}

bool RawConfigurationSystemID::setDAQSystemVersion(uint32_t isystem, uint32_t w)  {
    if(isystem == 0){
        std::cout<<"\nWARNING from RawConfigurationSystemID::setDAQSystemVersion! "
                 <<"That the hell? Why are you requesting the 0-th system?\n"
                 <<"Oh I see. You forgot that we count everything from 1... Check your code.\n";
        return false;
    }


    uint32_t word = (isystem - 1) * SYSTEM_SIZE;

    if(isystem > _SystemNumber)_SystemNumber = isystem;

    BITFIELDSET(RAWBUFF32[word], w, MASK_DAQSystemVersion, SHIFT_DAQSystemVersion);
    return true;
}
bool RawConfigurationSystemID::setDAQSystemID(uint32_t isystem, uint32_t w) {
    if(isystem == 0){
        std::cout<<"\nWARNING from RawConfigurationSystemID::setDAQSystemID! "
                 <<"That the hell? Why are you requesting the 0-th system?\n"
                 <<"Oh I see. You forgot that we count everything from 1... Check your code.\n";
        return false;
    }


    uint32_t word = (isystem - 1) * SYSTEM_SIZE;
    if(isystem > _SystemNumber)_SystemNumber = isystem;

    BITFIELDSET(RAWBUFF32[word], w, MASK_DAQSystemID, SHIFT_DAQSystemID);
    return true;
}


//------------------------------------------------------------------------------
// Printing the rawconfigurationheader structure
void RawConfigurationSystemID::print() const{
    printf("\n---RawConfigurationSystemID---------------------------------------------size = %u words\n",this->sizeofdata());

    for(uint32_t isystem = 1; isystem<= this->getSystemNumber(); ++isystem){
        printf("DAQSystemID(%u)=%u ", isystem, this->getDAQSystemID(isystem));
        printf("DAQSystemVersion(%u)=%u \n", isystem, this->getDAQSystemVersion(isystem));
    }
}

//------------------------------------------------------------------------------
// Size of the rawconfigurationheader _Buffer in words
uint32_t RawConfigurationSystemID::sizeofdata() const {

    return _SystemNumber * SYSTEM_SIZE;
}
