#include "DAQDataFormats/RawConfigurationTail.h"
#include "DAQDataFormats/DataFormatException.h"
#include <boost/lexical_cast.hpp>
#include <stdio.h>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif

using namespace daqdataformats;
using namespace rawconftail;
//------------------------------------------------------------------------------
// Default constructor
RawConfigurationTail::RawConfigurationTail(){

    /// Set pointers to point to the internal buffer
    this->resetBufferSource();

    this->init();
}
//------------------------------------------------------------------------------
RawConfigurationTail::RawConfigurationTail(const RawConfigurationTail& copy_in){
   this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawConfigurationTail::RawConfigurationTail(void* RawConfigurationTail_buffer){

    /// First, initialize the Internal Buffer
            /// Set pointers to point to the internal buffer
            this->resetBufferSource();

            /// Initialize it
            this->init();

    /// Now set the pointer to the external buffer
    this->setBufferSource(RawConfigurationTail_buffer);

}
//------------------------------------------------------------------------------
// Destructor
RawConfigurationTail::~RawConfigurationTail() { }

//------------------------------------------------------------------------------
// Initialization of Data members
void RawConfigurationTail::init(){

    /// Zero out first words
    for(uint32_t i = 0; i < WORD_CRC; ++i)
        _InternalBuffer[i] = 0;

    this->setMarkerLo1(Marker_value[0]);
    this->setMarkerLo2(Marker_value[1]);
}

//------------------------------------------------------------------------------
void RawConfigurationTail::copy(const RawConfigurationTail& copy_in){


      // Copy the data from one buffer to the other
    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // Setup the main buffer pointer
    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in._Buffer);
}

uint32_t RawConfigurationTail::getMarkerLo1()                  const {
        return BITFIELDGET(RAWBUFF32[WORD_MarkerLo1], MASK_MarkerLo1, SHIFT_MarkerLo1);
}
uint32_t RawConfigurationTail::getMarkerLo2()                  const {
        return BITFIELDGET(RAWBUFF32[WORD_MarkerLo2], MASK_MarkerLo2, SHIFT_MarkerLo2);
}
uint32_t RawConfigurationTail::getCRC()                  const {
        return BITFIELDGET(RAWBUFF32[WORD_CRC], MASK_CRC, SHIFT_CRC);
}

bool RawConfigurationTail::setMarkerLo1(uint32_t w) {

        BITFIELDSET(RAWBUFF32[WORD_MarkerLo1], w, MASK_MarkerLo1, SHIFT_MarkerLo1);
        return true;
}
bool RawConfigurationTail::setMarkerLo2(uint32_t w) {
        BITFIELDSET(RAWBUFF32[WORD_MarkerLo2], w, MASK_MarkerLo2, SHIFT_MarkerLo2);
        return true;
}
bool RawConfigurationTail::setCRC(uint32_t w) {
        BITFIELDSET(RAWBUFF32[WORD_CRC], w, MASK_CRC, SHIFT_CRC);
        return true;
}


bool RawConfigurationTail::setBufferSource(void* pointer){
        _Buffer = pointer;
        return true;
}
bool RawConfigurationTail::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
}

// clears all data in the buffer
bool RawConfigurationTail::clear(){

        /// Loop over the buffer and zero it
        for(uint32_t i = 0; i < this->sizeofdata(); ++i)
            RAWBUFF32[i] = 0;

        return true;
}
bool RawConfigurationTail::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
}
bool RawConfigurationTail::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
}



//------------------------------------------------------------------------------
// Printing the RawConfigurationTail structure
void RawConfigurationTail::print() const{
    printf("\n---RawConfigurationTail---------------------------------------------size = %u words\n",this->sizeofdata());

    printf(" MarkerLo1=%u", this->getMarkerLo1());
    printf(" MarkerLo2=%u", this->getMarkerLo2());
   
    std::cout<<"\nRawConfigurationTail Data RAW 32bit WORDs (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        this->printWord(i,false);
        printf(" ");
    }

    std::cout<<"\nRawConfigurationTail Data RAW 32bit Bits (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
            std::cout<<"\n    ";
            this->printWord(i,true);
    }

}

//------------------------------------------------------------------------------
// Size of the RawConfigurationTail _Buffer in words
uint32_t RawConfigurationTail::sizeofdata() const {

    return RAWCONFIGURATIONTAIL_SIZE;
}
