#include "DAQDataFormats/RawDCMChan.h"
#include <cstdio>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif

using namespace daqdataformats;
using namespace rawmillidcm;

//------------------------------------------------------------------------------
// Default constructor
RawDCMChan::RawDCMChan() {
    // Set pointers to point to the internal buffer
    this->resetBufferSource();

    // Initialize it
    this->init();
}
//------------------------------------------------------------------------------
RawDCMChan::RawDCMChan(const RawDCMChan& copy_in){
    this->copy(copy_in);
}


//------------------------------------------------------------------------------
RawDCMChan::RawDCMChan(void* dcmchan_buffer){
    // First, initialize the Internal Buffer
            // Set pointers to point to the internal buffer
            this->resetBufferSource();

            // Initialize it
            this->init();

    // Now set the pointer to the external buffer
    this->setBufferSource(dcmchan_buffer);

}

//------------------------------------------------------------------------------
// Destructor
RawDCMChan::~RawDCMChan() { }

//------------------------------------------------------------------------------
// Initialize the data members
void RawDCMChan::init(){

    for(uint32_t i = 0; i < this->sizeofdata(); i++)
        RAWBUFF32[i] = 0;

}
  //------------------------------------------------------------------------------
  void RawDCMChan::copy(const RawDCMChan& copy_in){

      // Copy the data from one buffer to the other
    for(int i = 0; i < RAWDCMCHAN_SIZE; ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // Setup the main buffer pointer
    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in._Buffer);

  }


//------------------------------------------------------------------------------
// Getters
    uint32_t RawDCMChan::getDiblockId() const  { return BITFIELDGET(RAWBUFF32[0], MASK_DiblockId  , SHIFT_DiblockId); }
    uint32_t RawDCMChan::getDCMId() const      { return BITFIELDGET(RAWBUFF32[0], MASK_DCMId      , SHIFT_DCMId); }
    uint32_t RawDCMChan::getDetId() const      { return BITFIELDGET(RAWBUFF32[0], MASK_DetId      , SHIFT_DetId); }
    uint32_t RawDCMChan::getGlobalDCMID() const{ return BITFIELDGET(RAWBUFF32[0], MASK_GlobalDCMID, SHIFT_GlobalDCMID); }
//------------------------------------------------------------------------------
// Setters
    bool RawDCMChan::setDiblockId(uint32_t id) {
        BITFIELDSET(RAWBUFF32[0], id, MASK_DiblockId  , SHIFT_DiblockId);
        return true;
    }
    bool RawDCMChan::setDCMId    (uint32_t id) {
        BITFIELDSET(RAWBUFF32[0], id, MASK_DCMId      , SHIFT_DCMId);
        return true;
    }
    bool RawDCMChan::setDetId    (uint32_t id) {
        BITFIELDSET(RAWBUFF32[0], id, MASK_DetId      , SHIFT_DetId);
        return true;
    }
    bool RawDCMChan::setGlobalDCMID(uint32_t id) {
        BITFIELDSET(RAWBUFF32[0], id, MASK_GlobalDCMID, SHIFT_GlobalDCMID);
        return true;
    }



    bool RawDCMChan::setBufferSource(void* pointer){
        _Buffer = pointer;
        return true;
    }
    bool RawDCMChan::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
    }

// clears all data in the buffer
    bool RawDCMChan::clear(){
        int max_data = this->sizeofdata();

        // Loop over the buffer and zero it
        for(int i=0; i< max_data; ++i)
            RAWBUFF32[i]=0;

        return true;
    }
    bool RawDCMChan::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
    }
    bool RawDCMChan::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
  }


//------------------------------------------------------------------------------
// Print the MilliSlice DCM Channel identification information
void RawDCMChan::print() const{
    printf(" DiblockId=%u DCMId=%u DetId=%u  RAW=",this->getDiblockId(),
                                                   this->getDCMId(),
                                                   this->getDetId());
    this->printWord(0,false);
    printf("\n");
    this->printWord(0,true);
}

//------------------------------------------------------------------------------
// Size of the _Buffer in words
uint32_t RawDCMChan::sizeofdata()const {return RAWDCMCHAN_SIZE;}

