#include "DAQDataFormats/RawEventTail.h"
#include <cstdio>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif


using namespace daqdataformats;
using namespace raweventtail;

//------------------------------------------------------------
// Default constructor
RawEventTail::RawEventTail(){
    _Buffer = _InternalBuffer;

    this->init();
}

//------------------------------------------------------------------------------
RawEventTail::RawEventTail(RawEventTail& copy_in){
    _Buffer = _InternalBuffer;
    this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawEventTail::RawEventTail(RawEventTail* copy_in){
    _Buffer = _InternalBuffer;
    this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawEventTail::RawEventTail(void* eventtail_buffer){
    this->setBufferSource(eventtail_buffer);
}

//------------------------------------------------------------
// Destructor
RawEventTail::~RawEventTail() { }

//------------------------------------------------------------
// Initialize the data members
void RawEventTail::init(){

    // Zero out the internal Buffer
    for(uint32_t i = 0; i < this->sizeofdata(); i++)
        RAWBUFF32[i] = 0;

    // Setup the EventTail markers
    this->setMarker(Marker_value[0]);
    this->setMarkerLeft(Marker_value[1]);
    this->setMarkerRight(Marker_value[2]);
}
//------------------------------------------------------------------------------
void RawEventTail::copy(RawEventTail& copy_in){
    // read the Buffer
    this->readData(copy_in.getBuffer());

}

//------------------------------------------------------------------------------
void RawEventTail::copy(RawEventTail* copy_in){

    this->readData(copy_in->getBuffer());

}

//------------------------------------------------------------------------------
void RawEventTail::copy(void* copy_in){
    // Copy the Buffer
    this->readData(copy_in);
}


//------------------------------------------------------------------------------
// Getters
    uint32_t RawEventTail::getMarker()        const {return BITFIELDGET(RAWBUFF32[WORD_Marker]       , MASK_Marker       , SHIFT_Marker);}
    uint32_t RawEventTail::getMarkerLeft()    const {return BITFIELDGET(RAWBUFF32[WORD_MarkerLeft]   , MASK_MarkerLeft   , SHIFT_MarkerLeft);}
    uint32_t RawEventTail::getMarkerRight()   const {return BITFIELDGET(RAWBUFF32[WORD_MarkerRight]  , MASK_MarkerRight  , SHIFT_MarkerRight);}
    uint32_t RawEventTail::getEventNumberLo() const {return BITFIELDGET(RAWBUFF32[WORD_EventNumberLo], MASK_EventNumberLo, SHIFT_EventNumberLo);}
    uint32_t RawEventTail::getEventNumberHi() const {return BITFIELDGET(RAWBUFF32[WORD_EventNumberHi], MASK_EventNumberHi, SHIFT_EventNumberHi);}
    uint32_t RawEventTail::getEventSize()     const {return BITFIELDGET(RAWBUFF32[WORD_EventSize]    , MASK_EventSize    , SHIFT_EventSize);}


    bool RawEventTail::setMarker        (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_Marker]       , inp, MASK_Marker       , SHIFT_Marker);
        return true;
    }
    bool RawEventTail::setMarkerLeft    (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_MarkerLeft]   , inp, MASK_MarkerLeft   , SHIFT_MarkerLeft);
        return true;
    }
    bool RawEventTail::setMarkerRight   (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_MarkerRight]  , inp, MASK_MarkerRight  , SHIFT_MarkerRight);
        return true;
    }
    bool RawEventTail::setEventNumberLo (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_EventNumberLo], inp, MASK_EventNumberLo, SHIFT_EventNumberLo);
        return true;
    }
    bool RawEventTail::setEventNumberHi (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_EventNumberHi], inp, MASK_EventNumberHi, SHIFT_EventNumberHi);
        return true;
    }
    bool RawEventTail::setEventSize     (uint32_t inp) {
        BITFIELDSET(RAWBUFF32[WORD_EventSize]    , inp, MASK_EventSize    , SHIFT_EventSize);
        return true;
    }

    bool RawEventTail::setBufferSource(void* pointer){
        _Buffer  = pointer;
        return true;
    }
    bool RawEventTail::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
    }

// clears all data in the buffer
    bool RawEventTail::clear(){
        int max_data = this->sizeofdata();

        // Loop over the buffer and zero it
        for(int i=0; i< max_data; ++i)
            RAWBUFF32[i]=0;

        return true;
    }
    bool RawEventTail::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
    }
    bool RawEventTail::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
  }


//------------------------------------------------------------------------------
// Print the RawEventTail header information
void RawEventTail::print() const{
    printf("\nRawEvent Tail:");
    printf(" Marker=%u"         , this->getMarker());       //! RawEventTail header delimiter
    printf(" EventNumberLo=%u"  , this->getEventNumberLo());
    printf(" EventNumberHi=%u"  , this->getEventNumberHi());
    printf(" MarkerLeft=%u"     , this->getMarkerLeft());       //! RawEventTail header delimiter
    printf(" MarkerRight=%u"    , this->getMarkerRight());       //! RawEventTail header delimiter
    printf(" EventSize=%u"      , this->getEventSize());
   
    std::cout<<"\nRAW:";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf(" ");
        this->printWord(i,false);
    }

    std::cout<<"\nRAW:";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf("\n");
        this->printWord(i,true);
    }

}

//------------------------------------------------------------------------------
// Size of _Buffer in words
uint32_t RawEventTail::sizeofdata()const {return RAWEVENTTAIL_SIZE;}
