#include "DAQDataFormats/RawMilliSliceHeader.h"
#include <cstdio>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif


using namespace daqdataformats;
using namespace rawmillihdr;

//------------------------------------------------------------
// Default constructor
RawMilliSliceHeader::RawMilliSliceHeader(){
    // Set the main buffer pointer to the internal buffer
    this->resetBufferSource();
    // Initialize it
    this->init();
}

//------------------------------------------------------------------------------
RawMilliSliceHeader::RawMilliSliceHeader(const RawMilliSliceHeader& copy_in){
    this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawMilliSliceHeader::RawMilliSliceHeader(void* millisliceheader_buffer){
    // First, initialize the Internal Buffer
            // Set pointers to point to the internal buffer
            this->resetBufferSource();

            // Initialize it
            this->init();

    // Now set the pointer to the external buffer
    this->setBufferSource(millisliceheader_buffer);

}

//------------------------------------------------------------
// Destructor
RawMilliSliceHeader::~RawMilliSliceHeader() { }

//------------------------------------------------------------
// Initialize the data members
void RawMilliSliceHeader::init(){

    for(uint32_t i = 0; i < this->sizeofdata(); i++)
        RAWBUFF32[i] = 0;

    // Set the MilliSlice Marker
    this->setMarker(Marker_value[0]);
    this->setSliceSize(MILLISLICEHEADER_SIZE);
}
//------------------------------------------------------------------------------
void RawMilliSliceHeader::copy(const RawMilliSliceHeader& copy_in){

    // Copy the data from one buffer to the other
    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in.getBuffer());
}

//------------------------------------------------------------------------------
// Getters

    RawDCMChan* RawMilliSliceHeader::getDCMChan()                     { return &_DCMChan; }
    uint32_t    RawMilliSliceHeader::getVersion() const               { return BITFIELDGET(RAWBUFF32[WORD_Version]            , MASK_Version            , SHIFT_Version); }
    uint32_t    RawMilliSliceHeader::getMarker() const                { return BITFIELDGET(RAWBUFF32[WORD_Marker]             , MASK_Marker             , SHIFT_Marker);}
    uint32_t    RawMilliSliceHeader::getDestBuffID() const            { return BITFIELDGET(RAWBUFF32[WORD_DestBuffID]         , MASK_DestBuffID         , SHIFT_DestBuffID); }
    uint32_t    RawMilliSliceHeader::getPartition() const             { return BITFIELDGET(RAWBUFF32[WORD_Partition]          , MASK_Partition          , SHIFT_Partition); }
    uint32_t    RawMilliSliceHeader::getTimeStartLow() const          { return BITFIELDGET(RAWBUFF32[WORD_TimeStartLow]       , MASK_TimeStartLow       , SHIFT_TimeStartLow); }
    uint32_t    RawMilliSliceHeader::getTimeStartHigh() const         { return BITFIELDGET(RAWBUFF32[WORD_TimeStartHigh]      , MASK_TimeStartHigh      , SHIFT_TimeStartHigh); }
    uint32_t    RawMilliSliceHeader::getTimeLength() const            { return BITFIELDGET(RAWBUFF32[WORD_TimeLength]         , MASK_TimeLength         , SHIFT_TimeLength); }
    uint32_t    RawMilliSliceHeader::getMilliSliceSEQ() const         { return BITFIELDGET(RAWBUFF32[WORD_SliceSEQ]           , MASK_SliceSEQ           , SHIFT_SliceSEQ); }
    uint32_t    RawMilliSliceHeader::getFrameSEQ() const              { return BITFIELDGET(RAWBUFF32[WORD_FrameSEQ]           , MASK_FrameSEQ           , SHIFT_FrameSEQ); }
    uint32_t    RawMilliSliceHeader::getSliceSize() const             { return BITFIELDGET(RAWBUFF32[WORD_SliceSize]          , MASK_SliceSize          , SHIFT_SliceSize); }
    uint32_t    RawMilliSliceHeader::getMicrosliceNumber() const      { return BITFIELDGET(RAWBUFF32[WORD_NumberOfMicroSlices], MASK_NumberOfMicroSlices, SHIFT_NumberOfMicroSlices);}
    uint32_t    RawMilliSliceHeader::getMilliSliceIndexNumber() const { return BITFIELDGET(RAWBUFF32[WORD_MilliSliceIndexNumber], MASK_MilliSliceIndexNumber, SHIFT_MilliSliceIndexNumber);}
    uint32_t    RawMilliSliceHeader::getMicroSliceTimeLength_usec() const { return BITFIELDGET(RAWBUFF32[WORD_MicroSliceTimeLength], MASK_MicroSliceTimeLength, SHIFT_MicroSliceTimeLength);}
    uint32_t    RawMilliSliceHeader::getDiblockId() const  { return _DCMChan.getDiblockId();}
    uint32_t    RawMilliSliceHeader::getDCMId()  const     { return _DCMChan.getDCMId();}
    uint32_t    RawMilliSliceHeader::getDetId() const      { return _DCMChan.getDetId();}
    uint32_t    RawMilliSliceHeader::getGlobalDCMID() const{ return _DCMChan.getGlobalDCMID();}
    Mode_t      RawMilliSliceHeader::getMode() const {
        Mode_t mode;
        uint32_t mode_uint32 =  BITFIELDGET(RAWBUFF32[WORD_Mode], MASK_Mode, SHIFT_Mode);
        mode = (Mode_t)mode_uint32;
        return mode;
    }
    bool        RawMilliSliceHeader::isComplete() const {
        if(BITFIELDGET(RAWBUFF32[WORD_Complete]                , MASK_Complete                , SHIFT_Complete))
            return true;
        return false;
    }
    bool        RawMilliSliceHeader::isConnectionInitialization() const {
        if(BITFIELDGET(RAWBUFF32[WORD_ConnectionInitialization], MASK_ConnectionInitialization, SHIFT_ConnectionInitialization))
            return true;
        return false;
    }
    bool        RawMilliSliceHeader::isCloseConnection() const {
        if(BITFIELDGET(RAWBUFF32[WORD_CloseConnection]         , MASK_CloseConnection         , SHIFT_CloseConnection))
            return true;
        return false;
    }
    bool        RawMilliSliceHeader::getMonteCarloFlag() const {
        if(BITFIELDGET(RAWBUFF32[WORD_MonteCarloFlag]         , MASK_MonteCarloFlag         , SHIFT_MonteCarloFlag))
            return true;
        return false;
    }
    bool        RawMilliSliceHeader::isCRCCalculationUsed() const {
        if(BITFIELDGET(RAWBUFF32[WORD_CRCCalculationUsed]     , MASK_CRCCalculationUsed     , SHIFT_CRCCalculationUsed))
            return true;
        return false;
    }


    bool        RawMilliSliceHeader::isMilliSliceIndexUsed() const {
        if(BITFIELDGET(RAWBUFF32[WORD_MilliSliceIndexUsed]     , MASK_MilliSliceIndexUsed     , SHIFT_MilliSliceIndexUsed))
            return true;
        return false;
    }


    uint64_t  RawMilliSliceHeader::getTimeStart()  const {
      uint64_t timestamp;
      uint32_t low  = this->getTimeStartLow();
      uint32_t high = this->getTimeStartHigh();

      timestamp = (uint64_t)high<<32 | low;
      return timestamp;
    }


//------------------------------------------------------------------------------
// Setters
    bool RawMilliSliceHeader::setDiblockId  (uint32_t id) { return _DCMChan.setDiblockId(id);}
    bool RawMilliSliceHeader::setDCMId      (uint32_t id) { return _DCMChan.setDCMId(id);}
    bool RawMilliSliceHeader::setDetId      (uint32_t id) { return _DCMChan.setDetId(id);}
    bool RawMilliSliceHeader::setGlobalDCMID(uint32_t id) { return _DCMChan.setGlobalDCMID(id);}
    bool RawMilliSliceHeader::setVersion   (uint32_t v) {
        BITFIELDSET(RAWBUFF32[WORD_Version]            , v  , MASK_Version            , SHIFT_Version);
        return true;
    }
    bool RawMilliSliceHeader::setMarker          (uint32_t m) {
        BITFIELDSET(RAWBUFF32[WORD_Marker]             , m  , MASK_Marker             , SHIFT_Marker);
        return true;
    }
    bool RawMilliSliceHeader::setDestBuffID      (uint32_t b) {
        BITFIELDSET(RAWBUFF32[WORD_DestBuffID]         , b  , MASK_DestBuffID         , SHIFT_DestBuffID);
        return true;
    }
    bool RawMilliSliceHeader::setPartition       (uint32_t p) {
        BITFIELDSET(RAWBUFF32[WORD_Partition]          , p  , MASK_Partition          , SHIFT_Partition);
        return true;
    }
    bool RawMilliSliceHeader::setTimeStartLow    (uint32_t ts) {
        BITFIELDSET(RAWBUFF32[WORD_TimeStartLow]       , ts , MASK_TimeStartLow       , SHIFT_TimeStartLow);
        return true;
    }
    bool RawMilliSliceHeader::setTimeStartHigh   (uint32_t ts) {
        BITFIELDSET(RAWBUFF32[WORD_TimeStartHigh]      , ts , MASK_TimeStartHigh      , SHIFT_TimeStartHigh);
        return true;
    }
    bool RawMilliSliceHeader::setTimeLength      (uint32_t tl) {
        BITFIELDSET(RAWBUFF32[WORD_TimeLength]         , tl , MASK_TimeLength         , SHIFT_TimeLength);
        return true;
    }
    bool RawMilliSliceHeader::setMilliSliceSEQ   (uint32_t seq) {
        BITFIELDSET(RAWBUFF32[WORD_SliceSEQ]           , seq, MASK_SliceSEQ           , SHIFT_SliceSEQ);
        return true;
    }
    bool RawMilliSliceHeader::setFrameSEQ        (uint32_t seq) {
        BITFIELDSET(RAWBUFF32[WORD_FrameSEQ]           , seq, MASK_FrameSEQ           , SHIFT_FrameSEQ);
        return true;
    }
    bool RawMilliSliceHeader::setSliceSize       (uint32_t s)  {
        BITFIELDSET(RAWBUFF32[WORD_SliceSize]          , s  , MASK_SliceSize          , SHIFT_SliceSize);
        return true;
    }
    bool RawMilliSliceHeader::setMicrosliceNumber(uint32_t n) {
        BITFIELDSET(RAWBUFF32[WORD_NumberOfMicroSlices], n  , MASK_NumberOfMicroSlices, SHIFT_NumberOfMicroSlices);
        return true;
    }

    bool RawMilliSliceHeader::setComplete                 (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_Complete]                , 0x1 , MASK_Complete                , SHIFT_Complete);
        else  BITFIELDSET(RAWBUFF32[WORD_Complete]                , 0x0 , MASK_Complete                , SHIFT_Complete);
        return true;
    }
    bool RawMilliSliceHeader::setMode                     (Mode_t mode){
        if(mode)BITFIELDSET(RAWBUFF32[WORD_Mode]                    , 0x1 , MASK_Mode                    , SHIFT_Mode);
        else  BITFIELDSET(RAWBUFF32[WORD_Mode]                    , 0x0 , MASK_Mode                    , SHIFT_Mode);
        return true;
    }
    bool RawMilliSliceHeader::setConnectionInitialization (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_ConnectionInitialization], 0x1 , MASK_ConnectionInitialization, SHIFT_ConnectionInitialization);
        else  BITFIELDSET(RAWBUFF32[WORD_ConnectionInitialization], 0x0 , MASK_ConnectionInitialization, SHIFT_ConnectionInitialization);
        return true;
    }
    bool RawMilliSliceHeader::setCloseConnection (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_CloseConnection], 0x1 , MASK_CloseConnection, SHIFT_CloseConnection);
        else  BITFIELDSET(RAWBUFF32[WORD_CloseConnection], 0x0 , MASK_CloseConnection, SHIFT_CloseConnection);
        return true;
    }

    bool RawMilliSliceHeader::setMilliSliceIndexNumber(uint32_t n) {
        BITFIELDSET(RAWBUFF32[WORD_MilliSliceIndexNumber], n  , MASK_MilliSliceIndexNumber, SHIFT_MilliSliceIndexNumber);
        return true;
    }

    bool RawMilliSliceHeader::setMicroSliceTimeLength_usec(uint32_t n) {
        BITFIELDSET(RAWBUFF32[WORD_MicroSliceTimeLength], n  , MASK_MicroSliceTimeLength, SHIFT_MicroSliceTimeLength);
        return true;
    }

    bool RawMilliSliceHeader::setMonteCarloFlag (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_MonteCarloFlag]    , 0x1 , MASK_MonteCarloFlag    , SHIFT_MonteCarloFlag);
        else  BITFIELDSET(RAWBUFF32[WORD_MonteCarloFlag]    , 0x0 , MASK_MonteCarloFlag    , SHIFT_MonteCarloFlag);
        return true;
    }
    bool RawMilliSliceHeader::setCRCCalculationUsed (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_CRCCalculationUsed], 0x1 , MASK_CRCCalculationUsed, SHIFT_CRCCalculationUsed);
        else  BITFIELDSET(RAWBUFF32[WORD_CRCCalculationUsed], 0x0 , MASK_CRCCalculationUsed, SHIFT_CRCCalculationUsed);
        return true;
    }

    bool RawMilliSliceHeader::setMilliSliceIndexUsed (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_MilliSliceIndexUsed], 0x1 , MASK_MilliSliceIndexUsed, SHIFT_MilliSliceIndexUsed);
        else  BITFIELDSET(RAWBUFF32[WORD_MilliSliceIndexUsed], 0x0 , MASK_MilliSliceIndexUsed, SHIFT_MilliSliceIndexUsed);
        return true;
    }

    bool RawMilliSliceHeader::advanceFrameSEQ()                    { 
        this->setFrameSEQ(this->getFrameSEQ() + 1);
        return true;
    }
    bool RawMilliSliceHeader::resetFrameSEQ()                      {
        this->setFrameSEQ(0);
        return true;
    }
    bool RawMilliSliceHeader::advanceMilliSliceSEQ()               {
        this->setMilliSliceSEQ(this->getMilliSliceSEQ() + 1);
        return true;
    }
    bool RawMilliSliceHeader::addSliceSize(uint32_t size)      {
        this->setSliceSize( this->getSliceSize() + size);
        return true;
    }
    bool RawMilliSliceHeader::advanceMicrosliceNumber()            {
        this->setMicrosliceNumber(this->getMicrosliceNumber() + 1);
        return true;
    }

    bool RawMilliSliceHeader::setTimeStart(uint64_t timestamp)  {
      uint32_t low, high;

     low  = timestamp & 0xffffffff;
     high = timestamp >> 32;

     this->setTimeStartHigh(high);
     this->setTimeStartLow(low);

     return true;
    }

    bool RawMilliSliceHeader::setBufferSource(void* pointer){
        _Buffer = pointer;
        uint32_t* pp = (uint32_t*)pointer + WORD_DCMChan;
        _DCMChan.setBufferSource(pp);
        return true;
}
    bool RawMilliSliceHeader::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
    }

// clears all data in the buffer
    bool RawMilliSliceHeader::clear(){
        int max_data = this->sizeofdata();

        // Loop over the buffer and zero it
        for(int i=0; i< max_data; ++i)
            RAWBUFF32[i]=0;

        return true;
    }
    bool RawMilliSliceHeader::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
    }
    bool RawMilliSliceHeader::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
  }


//------------------------------------------------------------------------------
// Print the MilliSlice header information
void RawMilliSliceHeader::print() const{
    printf("\nMilliSlice Header:");
    printf(" Version=%u"                   , this->getVersion());      //! Version number for the MilliSlice event builder
    printf(" Marker=0x%04x"                , this->getMarker());       //! MilliSlice header delimiter
    printf(" getDiblockId=%u"              , this->getDiblockId());
    printf(" DCMId=%u"                     , this->getDCMId());
    printf(" DetId=%u"                     , this->getDetId());
    printf(" DestBuff=%u"                  , this->getDestBuffID());   //! Destination buffer node
    printf(" Partition=%u"                 , this->getPartition());    //! Partition
    printf(" TimeStartLow=%u"              , this->getTimeStartLow()); //! Time start  0-31 bits for the MilliSlice
    printf(" TimeStartHigh=%u"             , this->getTimeStartHigh());//! Time start 32-55 bits for the MilliSlice
    printf(" TimeLength=%u"                , this->getTimeLength());   //! Time length of the MilliSlice
    printf(" isComplete=%u"                , this->isComplete());      //! Status word
    printf(" isMode=%u"                    , this->getMode());      //! Status word
    printf(" isConnectionInitialization=%u", this->isConnectionInitialization());      //! Status word
    printf(" isCloseConnection=%u"         , this->isCloseConnection());      //! Status word
    printf(" SliceSEQ=%u"                  , this->getMilliSliceSEQ());     //! MilliSlice sequence identifier
    printf(" FrameSEQ=%u"                  , this->getFrameSEQ());     //! Frame sequence identifier for current 5ms time slice
    printf(" SliceSize=%u"                 , this->getSliceSize());    //! The MilliSlice size in words excluding the header
    printf(" MicroSliceNumber=%u"          , this->getMicrosliceNumber()); //! Number of subframes in the millislice
    printf(" MilliSliceIndexNumber=%u"     , this->getMilliSliceIndexNumber());
    printf(" MicroSliceTimeLength_usec=%u" , this->getMicroSliceTimeLength_usec());
    printf(" MonteCarloFlag=%u"            , this->getMonteCarloFlag());
    printf(" isCRCCalculationUsed=%u"      , this->isCRCCalculationUsed());
    printf(" isMilliSliceIndexUsed=%u"     , this->isMilliSliceIndexUsed());

    std::cout<<"\nMilliSliceHeader RAW 32bit words (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf(" ");
        this->printWord(i, false);
    }

    std::cout<<"\nMilliSliceHeader RAW bits (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf("\n");
        this->printWord(i, true);
    }

}

//------------------------------------------------------------------------------
// Size of _Buffer in words
uint32_t RawMilliSliceHeader::sizeofdata()const {return MILLISLICEHEADER_SIZE;}
