#include "DAQDataFormats/RawNanoSliceHeader.h"
#include <cstdio>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif

using namespace daqdataformats;
using namespace rawnanohdr;
//------------------------Constructors------------------------------------------
RawNanoSliceHeader::RawNanoSliceHeader(){

    // Set pointers to point to the internal buffer
    this->resetBufferSource();
    
    // Initialize it
    this->init();
}

//------------------------------------------------------------------------------
RawNanoSliceHeader::RawNanoSliceHeader(NanoSliceHeader Header){

    // Set pointers to point to the internal buffer
    this->resetBufferSource();

    this->initWithData(Header);
}

//------------------------------------------------------------------------------
RawNanoSliceHeader::RawNanoSliceHeader(const RawNanoSliceHeader& copy_in){

    this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawNanoSliceHeader::RawNanoSliceHeader(void* nanosliceheader_buffer){

    // First, initialize the Internal Buffer
            // Set pointers to point to the internal buffer
            this->resetBufferSource();
    
            // Initialize it
            this->init();

    // Now set the pointer to the external buffer
    this->setBufferSource(nanosliceheader_buffer);
}

//------------------------Destructor--------------------------------------------
RawNanoSliceHeader::~RawNanoSliceHeader() { }

//------------------------------------------------------------------------------
// Default initialization at wherever the pointer points
void RawNanoSliceHeader::init(){

    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        RAWBUFF32[i] = 0;

    this->setPad3(1);
}

//------------------------------------------------------------------------------
// Initialization with data
 void RawNanoSliceHeader::initWithData(NanoSliceHeader Parameters){

    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        RAWBUFF32[i] = 0;

     this->setParameters(Parameters);
     this->setPad3(1);

 }


//------------------------------------------------------------------------------
// Deep Copy Method
void RawNanoSliceHeader::copy(const RawNanoSliceHeader& copy_in){
    // Copy the data from one buffer to the other
    for(int i = 0; i < RAWNANOSLICEHEADER_SIZE; ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // Setup the main buffer pointer
    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in._Buffer);


}

//------------------------------------------------------------------------------
// Getters
bool RawNanoSliceHeader::getDataPresent() const {
  return (BITFIELDGET(RAWBUFF32[0], MASK_DP, SHIFT_DP) ? true : false);
}

bool RawNanoSliceHeader::getLinkStatus() const  {
  return (BITFIELDGET(RAWBUFF32[0], MASK_LS, SHIFT_LS) ? true : false);
}

bool RawNanoSliceHeader::getDebugMode() const   {
  return (BITFIELDGET(RAWBUFF32[0], MASK_DM, SHIFT_DM) ? true : false);
}

uint32_t RawNanoSliceHeader::getFEBId() const     {
  return BITFIELDGET(RAWBUFF32[0], MASK_FEBId, SHIFT_FEBId);
}

uint32_t RawNanoSliceHeader::getFEBStatus() const {
  return BITFIELDGET(RAWBUFF32[0], MASK_FEBStatus, SHIFT_FEBStatus);
}

uint32_t RawNanoSliceHeader::getPixelAddr() const {
  return BITFIELDGET(RAWBUFF32[0], MASK_PixelAddress, SHIFT_PixelAddress);
}

uint32_t RawNanoSliceHeader::getSequence() const  {
  return BITFIELDGET(RAWBUFF32[0], MASK_SEQ, SHIFT_SEQ);
}

bool RawNanoSliceHeader::getDAQEnabled() const {
  return (BITFIELDGET(RAWBUFF32[0], MASK_DAQ_ENABLED, SHIFT_DAQ_ENABLED) ? true : false);
}

bool RawNanoSliceHeader::getBufferEmpty() const {
  return (BITFIELDGET(RAWBUFF32[0], MASK_BUFF_EMPTY, SHIFT_BUFF_EMPTY) ? true : false);
}

bool RawNanoSliceHeader::getBufferFull() const {
  return (BITFIELDGET(RAWBUFF32[0], MASK_BUFF_FULL, SHIFT_BUFF_FULL) ? true : false);
}

bool RawNanoSliceHeader::getCommError() const {
  return (BITFIELDGET(RAWBUFF32[0], MASK_COMM_ERR, SHIFT_COMM_ERR) ? true : false);
}

bool RawNanoSliceHeader::getPacketError() const {
  return (BITFIELDGET(RAWBUFF32[0], MASK_PAKT_ERR, SHIFT_PAKT_ERR) ? true : false);
}

bool RawNanoSliceHeader::getOverflowError() const {
  return (BITFIELDGET(RAWBUFF32[0], MASK_OVERFLOW_ERR, SHIFT_OVERFLOW_ERR) ? true : false);
}

bool RawNanoSliceHeader::getADCError() const {
  return (BITFIELDGET(RAWBUFF32[0], MASK_ADC_ERR, SHIFT_ADC_ERR) ? true : false);
}


    //------------------------------------------------------------------------------
//Setters
    bool RawNanoSliceHeader::setDataPresent (bool DataPresent)  {
        if(DataPresent) BITFIELDSET(RAWBUFF32[0], 0x1, MASK_DP , SHIFT_DP);
        else            BITFIELDSET(RAWBUFF32[0], 0x0, MASK_DP , SHIFT_DP);
        return true;
    }
    bool RawNanoSliceHeader::setLinkStatus  (bool LinkStatus)   {
        if(LinkStatus)  BITFIELDSET(RAWBUFF32[0], 0x1, MASK_LS , SHIFT_LS);
        else            BITFIELDSET(RAWBUFF32[0], 0x0, MASK_LS , SHIFT_LS);
        return true;
    }
    bool RawNanoSliceHeader::setDebugMode   (bool DebugMode) {
        if(DebugMode)   BITFIELDSET(RAWBUFF32[0], 0x1, MASK_DM , SHIFT_DM);
        else            BITFIELDSET(RAWBUFF32[0], 0x0, MASK_DM , SHIFT_DM);
        return true;
    }
    bool RawNanoSliceHeader::setFEBId       (uint32_t FEBId)  {
        BITFIELDSET(RAWBUFF32[0], FEBId,        MASK_FEBId,        SHIFT_FEBId);
        return true;
    }
    bool RawNanoSliceHeader::setFEBStatus   (uint32_t FEBStatus) {
        BITFIELDSET(RAWBUFF32[0], FEBStatus,    MASK_FEBStatus,    SHIFT_FEBStatus);
        return true;
    }
    bool RawNanoSliceHeader::setPixelAddr   (uint32_t PixelAddress) {
        BITFIELDSET(RAWBUFF32[0], PixelAddress, MASK_PixelAddress, SHIFT_PixelAddress);
        return true;
    }
    bool RawNanoSliceHeader::setSequence    (uint32_t SEQ)  {
        BITFIELDSET(RAWBUFF32[0], SEQ,          MASK_SEQ,          SHIFT_SEQ);
        return true;
    }
    bool RawNanoSliceHeader::setPad3        (uint32_t pad3) {
        BITFIELDSET(RAWBUFF32[0], pad3,           MASK_Pad3,         SHIFT_Pad3);
        return true;
    }
    bool RawNanoSliceHeader::advanceSequence() {return this->setSequence(this->getSequence() + 1);}

    bool RawNanoSliceHeader::setParameters(NanoSliceHeader Parameters){
        this->setLinkStatus(Parameters.LinkStatus);
        this->setDataPresent(Parameters.DataPresent);
        this->setDebugMode(Parameters.DebugMode);
        this->setFEBId(Parameters.FEBLinkID);
        this->setFEBStatus(Parameters.FEBStatus);
        this->setPixelAddr(Parameters.PixelAddress);
        this->setSequence(Parameters.FrameSequenceNumber);
        return true;
    }
    bool RawNanoSliceHeader::setBufferSource(void* pointer){
        _Buffer                = pointer;
        return true;
    }
    bool RawNanoSliceHeader::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
    }

// clears all data in the buffer
    bool RawNanoSliceHeader::clear(){
        int max_data = this->sizeofdata();

        // Loop over the buffer and zero it
        for(int i=0; i< max_data; ++i)
            RAWBUFF32[i]=0;

        return true;
    }
    bool RawNanoSliceHeader::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
    }
    bool RawNanoSliceHeader::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
  }

//------------------------------------------------------------------------------
// Printing method
void RawNanoSliceHeader::print() const{

    printf("NanoSlice Header: LS=%d DP=%d DM=%d FEBID=%u FEBSTAT=%u PIXEL=%u SEQ=%u RAW(hex)=",
                this->getLinkStatus(),
                this->getDataPresent(),
                this->getDebugMode(),
                this->getFEBId(),
                this->getFEBStatus(),
                this->getPixelAddr(),
                this->getSequence());
    this->printWord(0,false);
    printf("\n");

}

    //------------------------------------------------------------------------------
// Size of the MicroSlice Header in words
uint32_t RawNanoSliceHeader::sizeofdata()const {
    return RAWNANOSLICEHEADER_SIZE;
}
