#include "DAQDataFormats/RawRunHeader.h"
#include <cstdio>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif

using namespace daqdataformats;
using namespace rawrunhdr;
//------------------------Constructors------------------------------------------
RawRunHeader::RawRunHeader(){

    // Set pointers to point to the internal buffer
    this->resetBufferSource();
    
    // Initialize it
    this->init();
}

//------------------------------------------------------------------------------
RawRunHeader::RawRunHeader(const RawRunHeader& copy_in){

    this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawRunHeader::RawRunHeader(void* nanosliceheader_buffer){

    // First, initialize the Internal Buffer
            // Set pointers to point to the internal buffer
            this->resetBufferSource();
    
            // Initialize it
            this->init();

    // Now set the pointer to the external buffer
    this->setBufferSource(nanosliceheader_buffer);
}

//------------------------Destructor--------------------------------------------
RawRunHeader::~RawRunHeader() { }

//------------------------------------------------------------------------------
// Default initialization at wherever the pointer points
void RawRunHeader::init(){

    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        RAWBUFF32[i] = 0;

    this->setRunSize(RUNHEADER_SIZE);

    this->setMarkerHi(Marker_value[0]);
    this->setMarkerLo(Marker_value[1]);
}


//------------------------------------------------------------------------------
// Deep Copy Method
void RawRunHeader::copy(const RawRunHeader& copy_in){
    // Copy the data from one buffer to the other
    for(uint32_t i = 0; i < this->sizeofdata(); ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // Setup the main buffer pointer
    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in._Buffer);


}

bool RawRunHeader::close(){

    this->addCheckSum();
    return true;
}



//------------------------------------------------------------------------------
// Getters
    uint32_t RawRunHeader::getMarkerHi()               const {return BITFIELDGET(RAWBUFF32[WORD_MarkerHi]              , MASK_MarkerHi              , SHIFT_MarkerHi);}
    uint32_t RawRunHeader::getMarkerLo()               const {return BITFIELDGET(RAWBUFF32[WORD_MarkerLo]              , MASK_MarkerLo              , SHIFT_MarkerLo);}
    uint32_t RawRunHeader::getRunNumber()              const {return BITFIELDGET(RAWBUFF32[WORD_RunNumber]             , MASK_RunNumber             , SHIFT_RunNumber);}
    uint32_t RawRunHeader::getRunType()                const {return BITFIELDGET(RAWBUFF32[WORD_RunType]               , MASK_RunType               , SHIFT_RunType);}
    uint32_t RawRunHeader::getConfigIDX()              const {return BITFIELDGET(RAWBUFF32[WORD_ConfigIDX]             , MASK_ConfigIDX             , SHIFT_ConfigIDX);}
    uint32_t RawRunHeader::getPartition()              const {return BITFIELDGET(RAWBUFF32[WORD_Partition]             , MASK_Partition             , SHIFT_Partition);}
    uint32_t RawRunHeader::getSubrun()                 const {return BITFIELDGET(RAWBUFF32[WORD_Subrun]                , MASK_Subrun                , SHIFT_Subrun);}
    uint32_t RawRunHeader::getConfigurationBlockSet()  const {return BITFIELDGET(RAWBUFF32[WORD_ConfigurationBlockSet] , MASK_ConfigurationBlockSet , SHIFT_ConfigurationBlockSet);}
    uint32_t RawRunHeader::getRunControlVersion()      const {return BITFIELDGET(RAWBUFF32[WORD_RunControlVersion]     , MASK_RunControlVersion     , SHIFT_RunControlVersion);}
    uint32_t RawRunHeader::getRunControlID()           const {return BITFIELDGET(RAWBUFF32[WORD_RunControlID]          , MASK_RunControlID          , SHIFT_RunControlID);}
    uint32_t RawRunHeader::getDataLoggerVersion()      const {return BITFIELDGET(RAWBUFF32[WORD_DataLoggerVersion]     , MASK_DataLoggerVersion     , SHIFT_DataLoggerVersion);}
    uint32_t RawRunHeader::getDataLoggerID()           const {return BITFIELDGET(RAWBUFF32[WORD_DataLoggerID]          , MASK_DataLoggerID          , SHIFT_DataLoggerID);}
    uint32_t RawRunHeader::getRunStartTime()           const {return BITFIELDGET(RAWBUFF32[WORD_RunStartTime]          , MASK_RunStartTime          , SHIFT_RunStartTime);}
    uint32_t RawRunHeader::getSubRunStartTime ()       const {return BITFIELDGET(RAWBUFF32[WORD_SubRunStartTime]       , MASK_SubRunStartTime       , SHIFT_SubRunStartTime);}
    uint32_t RawRunHeader::getSubRunEndTime()          const {return BITFIELDGET(RAWBUFF32[WORD_SubRunEndTime]         , MASK_SubRunEndTime         , SHIFT_SubRunEndTime);}
    uint32_t RawRunHeader::getRunEndTime()             const {return BITFIELDGET(RAWBUFF32[WORD_RunEndTime]            , MASK_RunEndTime            , SHIFT_RunEndTime);}
    uint32_t RawRunHeader::getTotalEvents()            const {return BITFIELDGET(RAWBUFF32[WORD_TotalEvents]           , MASK_TotalEvents           , SHIFT_TotalEvents);}
    uint32_t RawRunHeader::getUniversalEventStartLo()  const {return BITFIELDGET(RAWBUFF32[WORD_UniversalEventStartLo] , MASK_UniversalEventStartLo , SHIFT_UniversalEventStartLo);}
    uint32_t RawRunHeader::getUniversalEventStartHi()  const {return BITFIELDGET(RAWBUFF32[WORD_UniversalEventStartHi] , MASK_UniversalEventStartHi , SHIFT_UniversalEventStartHi);}
    uint32_t RawRunHeader::getUniversalEventEndLo()    const {return BITFIELDGET(RAWBUFF32[WORD_UniversalEventEndLo]   , MASK_UniversalEventEndLo   , SHIFT_UniversalEventEndLo);}
    uint32_t RawRunHeader::getUniversalEventEndHi()    const {return BITFIELDGET(RAWBUFF32[WORD_UniversalEventEndHi]   , MASK_UniversalEventEndHi   , SHIFT_UniversalEventEndHi);}
    uint32_t RawRunHeader::getRunSize()                const {return BITFIELDGET(RAWBUFF32[WORD_RunSize]               , MASK_RunSize               , SHIFT_RunSize);}
    uint32_t RawRunHeader::getTriggerVersion()         const {return BITFIELDGET(RAWBUFF32[WORD_TriggerVersion]        , MASK_TriggerVersion        , SHIFT_TriggerVersion);}
    uint32_t RawRunHeader::getTriggerCtrlID()          const {return BITFIELDGET(RAWBUFF32[WORD_TriggerCtrlID]         , MASK_TriggerCtrlID         , SHIFT_TriggerCtrlID);}
    uint32_t RawRunHeader::getTriggerMC()              const {return BITFIELDGET(RAWBUFF32[WORD_TriggerMC]             , MASK_TriggerMC             , SHIFT_TriggerMC);}
    uint32_t RawRunHeader::getVTMod()                  const {return BITFIELDGET(RAWBUFF32[WORD_VTMod]                 , MASK_VTMod                 , SHIFT_VTMod);}
    uint32_t RawRunHeader::getValidTriggerTypesLow()   const {return BITFIELDGET(RAWBUFF32[WORD_ValidTriggerTypesLow]  , MASK_ValidTriggerTypesLow  , SHIFT_ValidTriggerTypesLow);}
    uint32_t RawRunHeader::getValidTriggerTypesHigh()  const {return BITFIELDGET(RAWBUFF32[WORD_ValidTriggerTypesHigh] , MASK_ValidTriggerTypesHigh , SHIFT_ValidTriggerTypesHigh);}
    uint32_t RawRunHeader::getValidTriggerTypesHigh2() const {return BITFIELDGET(RAWBUFF32[WORD_ValidTriggerTypesHigh2], MASK_ValidTriggerTypesHigh2, SHIFT_ValidTriggerTypesHigh2);}
    uint32_t RawRunHeader::getTriggerListIDX()         const {return BITFIELDGET(RAWBUFF32[WORD_TriggerListIDX]        , MASK_TriggerListIDX        , SHIFT_TriggerListIDX);}
    uint32_t RawRunHeader::getTriggerPrescaleListIDX() const {return BITFIELDGET(RAWBUFF32[WORD_TriggerPrescaleListIDX], MASK_TriggerPrescaleListIDX, SHIFT_TriggerPrescaleListIDX);}
    uint32_t RawRunHeader::getReserved1()              const {return BITFIELDGET(RAWBUFF32[WORD_Reserved1]             , MASK_Reserved1             , SHIFT_Reserved1);}
    uint32_t RawRunHeader::getReserved2()              const {return BITFIELDGET(RAWBUFF32[WORD_Reserved2]             , MASK_Reserved1             , SHIFT_Reserved2);}


uint64_t RawRunHeader::getUniversalEventStart() const {
      uint64_t trigstamp;
      uint32_t low, high;

      low  = this->getUniversalEventStartLo();
      high = this->getUniversalEventStartHi();

      trigstamp=(uint64_t)high<<32 | low;
      return trigstamp;
}
uint64_t RawRunHeader::getUniversalEventEnd() const {
      uint64_t trigstamp;
      uint32_t low, high;

      low  = this->getUniversalEventEndLo();
      high = this->getUniversalEventEndHi();

      trigstamp=(uint64_t)high<<32 | low;
      return trigstamp;
}
    


//------------------------------------------------------------------------------
//Setters
bool RawRunHeader::setMarkerHi              (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_MarkerHi], w, MASK_MarkerHi, SHIFT_MarkerHi);
    return true;
}
bool RawRunHeader::setMarkerLo              (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_MarkerLo], w, MASK_MarkerLo, SHIFT_MarkerLo);
    return true;
}
bool RawRunHeader::setRunNumber             (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_RunNumber], w, MASK_RunNumber, SHIFT_RunNumber);
    return true;
}
bool RawRunHeader::setRunType               (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_RunType], w, MASK_RunType, SHIFT_RunType);
    return true;
}
bool RawRunHeader::setConfigIDX             (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_ConfigIDX], w, MASK_ConfigIDX, SHIFT_ConfigIDX);
    return true;
}
bool RawRunHeader::setPartition             (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_Partition], w, MASK_Partition, SHIFT_Partition);
    return true;
}
bool RawRunHeader::setSubrun                (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_Subrun], w, MASK_Subrun, SHIFT_Subrun);
    return true;
}

bool RawRunHeader::setConfigurationBlockSet (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_ConfigurationBlockSet], w, MASK_ConfigurationBlockSet, SHIFT_ConfigurationBlockSet);
    return true;
}

bool RawRunHeader::setRunControlVersion     (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_RunControlVersion], w, MASK_RunControlVersion, SHIFT_RunControlVersion);
    return true;
}
bool RawRunHeader::setRunControlID          (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_RunControlID], w, MASK_RunControlID, SHIFT_RunControlID);
    return true;
}
bool RawRunHeader::setDataLoggerVersion     (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_DataLoggerVersion], w, MASK_DataLoggerVersion, SHIFT_DataLoggerVersion);
    return true;
}
bool RawRunHeader::setDataLoggerID          (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_DataLoggerID], w, MASK_DataLoggerID, SHIFT_DataLoggerID);
    return true;
}
bool RawRunHeader::setRunStartTime          (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_RunStartTime], w, MASK_RunStartTime, SHIFT_RunStartTime);
    return true;
}
bool RawRunHeader::setSubRunStartTime       (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_SubRunStartTime], w, MASK_SubRunStartTime, SHIFT_SubRunStartTime);
    return true;
}
bool RawRunHeader::setSubRunEndTime         (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_SubRunEndTime], w, MASK_SubRunEndTime, SHIFT_SubRunEndTime);
    return true;
}
bool RawRunHeader::setRunEndTime            (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_RunEndTime], w, MASK_RunEndTime, SHIFT_RunEndTime);
    return true;
}
bool RawRunHeader::setTotalEvents           (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_TotalEvents], w, MASK_TotalEvents, SHIFT_TotalEvents);
    return true;
}
bool RawRunHeader::setUniversalEventStartLo (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_UniversalEventStartLo], w, MASK_UniversalEventStartLo, SHIFT_UniversalEventStartLo);
    return true;
}
bool RawRunHeader::setUniversalEventStartHi (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_UniversalEventStartHi], w, MASK_UniversalEventStartHi, SHIFT_UniversalEventStartHi);
    return true;
}
bool RawRunHeader::setUniversalEventEndLo   (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_UniversalEventEndLo], w, MASK_UniversalEventEndLo, SHIFT_UniversalEventEndLo);
    return true;
}
bool RawRunHeader::setUniversalEventEndHi   (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_UniversalEventEndHi], w, MASK_UniversalEventEndHi, SHIFT_UniversalEventEndHi);
    return true;
}
bool RawRunHeader::setRunSize               (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_RunSize], w, MASK_RunSize, SHIFT_RunSize);
    return true;
}
bool RawRunHeader::setTriggerVersion        (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_TriggerVersion], w, MASK_TriggerVersion, SHIFT_TriggerVersion);
    return true;
}
bool RawRunHeader::setTriggerCtrlID         (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_TriggerCtrlID], w, MASK_TriggerCtrlID, SHIFT_TriggerCtrlID);
    return true;
}
bool RawRunHeader::setTriggerMC             (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_TriggerMC], w, MASK_TriggerMC, SHIFT_TriggerMC);
    return true;
}
bool RawRunHeader::setVTMod                 (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_VTMod], w, MASK_VTMod, SHIFT_VTMod);
    return true;
}
bool RawRunHeader::setValidTriggerTypesLow  (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_ValidTriggerTypesLow], w, MASK_ValidTriggerTypesLow, SHIFT_ValidTriggerTypesLow);
    return true;
}
bool RawRunHeader::setValidTriggerTypesHigh (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_ValidTriggerTypesHigh], w, MASK_ValidTriggerTypesHigh, SHIFT_ValidTriggerTypesHigh);
    return true;
}
bool RawRunHeader::setValidTriggerTypesHigh2(uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_ValidTriggerTypesHigh2], w, MASK_ValidTriggerTypesHigh2, SHIFT_ValidTriggerTypesHigh2);
    return true;
}
bool RawRunHeader::setTriggerListIDX        (uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_TriggerListIDX], w, MASK_TriggerListIDX, SHIFT_TriggerListIDX);
    return true;
}
bool RawRunHeader::setTriggerPrescaleListIDX(uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_TriggerPrescaleListIDX], w, MASK_TriggerPrescaleListIDX, SHIFT_TriggerPrescaleListIDX);
    return true;
}
bool RawRunHeader::setReserved1(uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_Reserved1], w, MASK_Reserved1, SHIFT_Reserved1);
    return true;
}
bool RawRunHeader::setReserved2(uint32_t w){
    BITFIELDSET(RAWBUFF32[WORD_Reserved2], w, MASK_Reserved2, SHIFT_Reserved2);
    return true;
}

bool RawRunHeader::setUniversalEventStart(uint64_t eventstamp) {

    uint32_t low, high;

    low  = eventstamp & 0xffffffff;
    high = eventstamp >> 32;

    this->setUniversalEventStartHi(high);
    this->setUniversalEventStartLo(low);

    return true;
}
bool RawRunHeader::setUniversalEventEnd(uint64_t eventstamp) {

    uint32_t low, high;

    low  = eventstamp & 0xffffffff;
    high = eventstamp >> 32;

    this->setUniversalEventEndHi(high);
    this->setUniversalEventEndLo(low);

    return true;
}

bool RawRunHeader::addValidTriggerTypesLow  (uint32_t w){

    uint32_t word = w | this->getValidTriggerTypesLow();

    BITFIELDSET(RAWBUFF32[WORD_ValidTriggerTypesLow], word, MASK_ValidTriggerTypesLow, SHIFT_ValidTriggerTypesLow);
    return true;
}
bool RawRunHeader::addValidTriggerTypesHigh (uint32_t w){

    uint32_t word = w | this->getValidTriggerTypesHigh();

    BITFIELDSET(RAWBUFF32[WORD_ValidTriggerTypesHigh], word, MASK_ValidTriggerTypesHigh, SHIFT_ValidTriggerTypesHigh);
    return true;
}
bool RawRunHeader::addValidTriggerTypesHigh2(uint32_t w){

    uint32_t word = w | this->getValidTriggerTypesHigh2();

    BITFIELDSET(RAWBUFF32[WORD_ValidTriggerTypesHigh2], word, MASK_ValidTriggerTypesHigh2, SHIFT_ValidTriggerTypesHigh2);
    return true;
}

bool RawRunHeader::addRunSize (uint32_t size){ return this->setRunSize(this->getRunSize() + size);}
bool RawRunHeader::advanceEventNumber()      { return this->setTotalEvents(this->getTotalEvents() + 1);}



bool RawRunHeader::setTail(){

       this->setMarkerHi(rawruntail::Marker_value[0]);
       this->setMarkerLo(rawruntail::Marker_value[1]);
       this->setReserved1(rawruntail::Marker_value[2]);
       this->setReserved2(rawruntail::Marker_value[3]);
       return true;
}





bool RawRunHeader::setBufferSource(void* pointer){
        _Buffer                = pointer;
        return true;
    }
bool RawRunHeader::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
    }

// clears all data in the buffer
bool RawRunHeader::clear(){
        int max_data = this->sizeofdata();

        // Loop over the buffer and zero it
        for(int i=0; i< max_data; ++i)
            RAWBUFF32[i]=0;

        return true;
    }
bool RawRunHeader::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
    }
bool RawRunHeader::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
  }

//------------------------------------------------------------------------------
// Printing method
void RawRunHeader::print() const{

    printf("RawRun Header--------------------------------------\n");

    printf(" RunNumber=%u",this->getRunNumber());
    printf(" RunType=%u",this->getRunType());
    printf(" ConfigIDX=%u",this->getConfigIDX());
    printf(" Partition=%u",this->getPartition());
    printf(" Subrun=%u",this->getSubrun());
    printf(" ConfigBlockSet=%u",this->getConfigurationBlockSet());
    printf(" RunControlVersion=%u",this->getRunControlVersion());
    printf(" RunControlID=%u",this->getRunControlID());
    printf(" DataLoggerVersion=%u",this->getDataLoggerVersion());
    printf(" DataLoggerID=%u",this->getDataLoggerID());
    printf(" RunStartTime=%u",this->getRunStartTime());
    printf(" SubRunStartTime=%u",this->getSubRunStartTime());
    printf(" SubRunEndTime=%u",this->getSubRunEndTime ());
    printf(" RunEndTime=%u",this->getRunEndTime());
    printf(" TotalEvents=%u",this->getTotalEvents());
    printf(" UniversalEventStartLo=%u",this->getUniversalEventStartLo());
    printf(" UniversalEventStartHi=%u",this->getUniversalEventStartHi());
    printf(" UniversalEventEndLo=%u",this->getUniversalEventEndLo());
    printf(" UniversalEventEndHi=%u",this->getUniversalEventEndHi());
    printf(" RunSize=%u",this->getRunSize());
    printf(" TriggerVersion=%u",this->getTriggerVersion());
    printf(" TriggerCtrlID=%u",this->getTriggerCtrlID());
    printf(" TriggerMC=%u",this->getTriggerMC());
    printf(" VTMod=%u",this->getVTMod());
    printf(" ValidTriggerTypesLow=%u",this->getValidTriggerTypesLow());
    printf(" ValidTriggerTypesHigh=%u",this->getValidTriggerTypesHigh());
    printf(" ValidTriggerTypesHigh2=%u",this->getValidTriggerTypesHigh2());
    printf(" TriggerListIDX=%u",this->getTriggerListIDX());
    printf(" TriggerPrescaleListIDX=%u",this->getTriggerPrescaleListIDX());

    std::cout<<"\nRAW:";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf(" ");
        this->printWord(i,false);
    }

    std::cout<<"\nRAW:";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
            if(i > 0)std::cout<<"    ";
            this->printWord(i,true);
            std::cout<<"\n";
            }

}

    //------------------------------------------------------------------------------
// Size of the MicroSlice Header in words
uint32_t RawRunHeader::sizeofdata()const {
    return RUNHEADER_SIZE;
}
